package com.rjconsultores.ventaboletos;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.flywaydb.core.Flyway;

public class FlyWay {
	private static Logger log = Logger.getLogger(FlyWay.class);
	
	private static final FlyWay INSTANCE = new FlyWay();
	
	private DataSource dataSource = null;
	
	private String url = null;
	private String user = null;
	private String password = null;
	
	private String location = "db.migration";
	
	public static FlyWay getInstance() {
		return INSTANCE;
	}
	
	public void defineLocation(final String location) {
		this.location = location;
	}
	
	public void defineDataSource(final DataSource dataSource) {
		this.dataSource = dataSource;
		
		fixFiles(dataSource);
	}
	
	public void defineProperties(final String url, final String user, final String password) {
		this.url = url;
		this.user = user;
		this.password = password;
	}
	
	private void fixFiles(final DataSource ds) {
		try {
			Connection c = dataSource.getConnection();
			Statement s = c.createStatement();

			fixVersion(s, "20161710.1833", "20161017.1833", "V20161017_1833__mantis8112.sql");
			fixVersion(s, "20161910.0934", "20161019.0934", "V20161019_0934__mantis7907.sql");
			fixVersion(s, "20162410.1119", "20161024.1119", "V20161024_1119__mantis7904.sql");

			s.close();
			
		} catch (SQLException e) {
			log.error(e.getMessage(), e);
		}
	}

	private void fixVersion(final Statement s, final String oldVersion, final String newVersion, final String newScript) throws SQLException {
		ResultSet rs = s.executeQuery("select count(*) as total from \"schema_version\" where \"version\" like '" + oldVersion + "'");
		
		int total = 0;
		
		while (rs.next()) {
			total = rs.getInt("total");
		}
		
		rs.close();
		
		if (total > 0) {
			log.info("Fix version " + oldVersion + " to new version " + newVersion + " and new script " + newScript);

			StringBuilder sql = new StringBuilder("");
			sql.append(" update \"schema_version\"");
			sql.append(" set \"version\" = '" + newVersion + "', \"script\" = '" + newScript + "'");
			sql.append(" where \"version\" like '" + oldVersion + "'");
			
			s.executeUpdate(sql.toString());
		}
	}

	public boolean start() {
		return start(null, null);
	}

	public boolean startCustom() {
		final File jbossData = new File(System.getProperty("jboss.server.data.dir"));
		final String table = "schema_version_perf";

		String location = "db.performance";
		
		if (jbossData != null) {
			File data = new File(jbossData, "flyway");
			
			if (!data.exists()) {
				data.mkdirs();
			}
			
			validarScripts(data);
			
			location = "filesystem:" + data.getPath();
		}
		
		return start(location, table);
	}

	public boolean start(final String customLocation, final String customTable) {
		log.info("Executando Flyway Custom...");
		
		try{
			final Flyway flyway = new Flyway();
			
			if (customTable != null) {
				flyway.setTable(customTable);
			}
			
//			FlyWayCallback fc = new FlyWayCallback();
//			flyway.setCallbacks(fc);
			
			if (customLocation != null) {
				flyway.setLocations(customLocation);
			} else if (this.location != null && !this.location.equals("db.migration")) {
				flyway.setLocations(this.location);
			}
			
			if (dataSource != null) {
				flyway.setDataSource(dataSource);
			} else {
				flyway.setDataSource(url, user, password);
			}
			
			execute(flyway);
			
		}catch(Throwable t){
			log.error("Erro ao executar o Flyway Custom",t);
			
			return false;
		}
		
		log.info("Flyway Custom executado.");
		
		return true;
	}
	
	public void execute(final Flyway flyway) {
		flyway.setValidateOnMigrate(false);
		flyway.setIgnoreFutureMigrations(true);
		flyway.setOutOfOrder(true);
		flyway.setBaselineOnMigrate(true);

		flyway.migrate();
	}
	
	private void validarScripts(File data) {
		Map<String, String> scripts = new HashMap<String, String>(0);
		
		try {
			Connection c = dataSource.getConnection();
			Statement s = c.createStatement();
			
			ResultSet rs = s.executeQuery("select nome as nome_arquivo, sql as sql_arquivo from flyway_scripts");
			
			while (rs.next()) {
				scripts.put(rs.getString("nome_arquivo"), rs.getString("sql_arquivo"));
			}
			
			rs.close();
			s.close();
			
		} catch (SQLException e) {
			log.error(e.getMessage(), e);
		}
		
		Writer writer = null;
		
		for (Map.Entry<String, String> entry : scripts.entrySet()) {
			File scriptFile = new File(data, entry.getKey() + ".sql");
			
			if (!scriptFile.exists()) {
				try {
				    writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(scriptFile), "utf-8"));
				    writer.write(entry.getValue());
				} catch (IOException ex) {
				} finally {
				   try {writer.close();} catch (Exception ex) {log.info(ex.getMessage(), ex);}
				}			
			}
		}
	}
}
